
-------------------------------------------------------------------------------
--
-- File: tb_TestTop_AllZmods.vhd
-- Author: Tudor Gherman, Robert Bocos
-- Original Project: Zmod ADC 1410 Low Level Controller
-- Date: 11 Dec. 2020
--
-------------------------------------------------------------------------------
-- (c) 2020 Copyright Digilent Incorporated
-- All Rights Reserved
-- 
-- This program is free software; distributed under the terms of BSD 3-clause 
-- license ("Revised BSD License", "New BSD License", or "Modified BSD License")
--
-- Redistribution and use in source and binary forms, with or without modification,
-- are permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, this
--    list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright notice,
--    this list of conditions and the following disclaimer in the documentation
--    and/or other materials provided with the distribution.
-- 3. Neither the name(s) of the above-listed copyright holder(s) nor the names
--    of its contributors may be used to endorse or promote products derived
--    from this software without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
-- FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
-- SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
-- CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
-- OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
--
-- This test bench is used to instantiate the tb_TestTop test bench for all
-- supported ZmodADC variants.
--  
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
use work.PkgZmodDigitizer.all;

entity tb_TestTop_AllZmods is

end tb_TestTop_AllZmods;

architecture Behavioral of tb_TestTop_AllZmods is

constant kADC_SamplingClkPeriod_122_88 : time := 8.138ns;
constant kADC_SamplingClkPeriod_50 : time := 20ns;
constant kADC_SamplingClkPeriod_80 : time := 12.500ns;
constant kADC_SamplingClkPeriod_100 : time := 10ns;
constant kADC_SamplingClkPeriod_110 : time := 9.090ns;
constant kADC_SamplingClkPeriod_120 : time := 8.333ns;
constant kADC_SamplingClkPeriod_125 : time := 8ns;
constant kExtCalibEn : boolean := true; 
constant kExtCmdInterfaceEn : boolean := true;
constant kCDCE_SimulationConfig : boolean := true;
constant kCDCE_SimulationCmdTotal : integer range 0 to kCDCE_RegNrZeroBased := 4;
constant kCDCEFreqSel_122_88 : integer range 0 to (kCDCE_FreqCfgsNr - 1) := 0;
constant kCDCEFreqSel_50 : integer range 0 to (kCDCE_FreqCfgsNr - 1) := 1;
constant kCDCEFreqSel_80 : integer range 0 to (kCDCE_FreqCfgsNr - 1) := 2;
constant kCDCEFreqSel_100 : integer range 0 to (kCDCE_FreqCfgsNr - 1) := 3;
constant kCDCEFreqSel_110 : integer range 0 to (kCDCE_FreqCfgsNr - 1) := 4;
constant kCDCEFreqSel_120 : integer range 0 to (kCDCE_FreqCfgsNr - 1) := 5;
constant kCDCEFreqSel_125 : integer range 0 to (kCDCE_FreqCfgsNr - 1) := 6;
constant kCDCEI2C_Addr : std_logic_vector(7 downto 0) := x"CE";     
constant kCh1LgMultCoefStatic : std_logic_vector (17 downto 0) := "010001101010110010";
constant kCh1LgAddCoefStatic : std_logic_vector (17 downto 0) := "111111101111010101";
constant kCh1HgMultCoefStatic : std_logic_vector (17 downto 0) := "010001101010100010";  
constant kCh1HgAddCoefStatic : std_logic_vector (17 downto 0) := "111111101111000101";  
constant kCh2LgMultCoefStatic : std_logic_vector (17 downto 0) := "010001101010010010";  
constant kCh2LgAddCoefStatic : std_logic_vector (17 downto 0) := "111111101101010101"; 
constant kCh2HgMultCoefStatic : std_logic_vector (17 downto 0) := "010001101000110010"; 
constant kCh2HgAddCoefStatic : std_logic_vector (17 downto 0) := "111111101111010001";    
     
begin

------------------------------------------------------------------------------------------
-- Top level test bench instantiated for all supported Zmod ADC variants
------------------------------------------------------------------------------------------ 

InstTbTestTop_ZmodDigitizer_122_88: entity work.tb_TestTop
   Generic Map(
      kZmodID => kZmodDigitizer1430_125,
      kADC_SamplingClkPeriod => kADC_SamplingClkPeriod_122_88,
      kADC_ClkDiv => 1,
      kExtCalibEn => kExtCalibEn, 
      kExtCmdInterfaceEn => kExtCmdInterfaceEn,  
      kCh1HgMultCoefStatic => kCh1HgMultCoefStatic,
      kCh1HgAddCoefStatic => kCh1HgAddCoefStatic,
      kCh2HgMultCoefStatic => kCh2HgMultCoefStatic,
      kCh2HgAddCoefStatic => kCh2HgAddCoefStatic,
      kCDCEI2C_Addr => kCDCEI2C_Addr,
      kCDCE_SimulationConfig => kCDCE_SimulationConfig,
      kCDCE_SimulationCmdTotal => kCDCE_SimulationCmdTotal,
      kCDCEFreqSel => kCDCEFreqSel_122_88
   );
   
InstTbTestTop_ZmodDigitizer_50: entity work.tb_TestTop
   Generic Map(
      kZmodID => kZmodDigitizer1430_125,
      kADC_SamplingClkPeriod => kADC_SamplingClkPeriod_50,
      kADC_ClkDiv => 1,
      kExtCalibEn => kExtCalibEn, 
      kExtCmdInterfaceEn => kExtCmdInterfaceEn,    
      kCh1HgMultCoefStatic => kCh1HgMultCoefStatic,
      kCh1HgAddCoefStatic => kCh1HgAddCoefStatic,
      kCh2HgMultCoefStatic => kCh2HgMultCoefStatic,
      kCh2HgAddCoefStatic => kCh2HgAddCoefStatic,
      kCDCEI2C_Addr => kCDCEI2C_Addr,
      kCDCE_SimulationConfig => kCDCE_SimulationConfig,
      kCDCE_SimulationCmdTotal => kCDCE_SimulationCmdTotal,
      kCDCEFreqSel => kCDCEFreqSel_50 
   );
   
InstTbTestTop_ZmodDigitizer_80: entity work.tb_TestTop
   Generic Map(
      kZmodID => kZmodDigitizer1430_125,
      kADC_SamplingClkPeriod => kADC_SamplingClkPeriod_80,
      kADC_ClkDiv => 1,
      kExtCalibEn => kExtCalibEn, 
      kExtCmdInterfaceEn => kExtCmdInterfaceEn,
      kCh1HgMultCoefStatic => kCh1HgMultCoefStatic,
      kCh1HgAddCoefStatic => kCh1HgAddCoefStatic,
      kCh2HgMultCoefStatic => kCh2HgMultCoefStatic,
      kCh2HgAddCoefStatic => kCh2HgAddCoefStatic,
      kCDCEI2C_Addr => kCDCEI2C_Addr,
      kCDCE_SimulationConfig => kCDCE_SimulationConfig,
      kCDCE_SimulationCmdTotal => kCDCE_SimulationCmdTotal,
      kCDCEFreqSel => kCDCEFreqSel_80 
   );         
   
InstTbTestTop_ZmodDigitizer_100: entity work.tb_TestTop
   Generic Map(
      kZmodID => kZmodDigitizer1430_125,
      kADC_SamplingClkPeriod => kADC_SamplingClkPeriod_100,
      kADC_ClkDiv => 1,
      kExtCalibEn => kExtCalibEn, 
      kExtCmdInterfaceEn => kExtCmdInterfaceEn,    
      kCh1HgMultCoefStatic => kCh1HgMultCoefStatic,
      kCh1HgAddCoefStatic => kCh1HgAddCoefStatic,
      kCh2HgMultCoefStatic => kCh2HgMultCoefStatic,
      kCh2HgAddCoefStatic => kCh2HgAddCoefStatic,
      kCDCEI2C_Addr => kCDCEI2C_Addr,
      kCDCE_SimulationConfig => kCDCE_SimulationConfig,
      kCDCE_SimulationCmdTotal => kCDCE_SimulationCmdTotal,
      kCDCEFreqSel => kCDCEFreqSel_100 
   );
   
InstTbTestTop_ZmodDigitizer_110: entity work.tb_TestTop
   Generic Map(
      kZmodID => kZmodDigitizer1430_125,
      kADC_SamplingClkPeriod => kADC_SamplingClkPeriod_110,
      kADC_ClkDiv => 1,
      kExtCalibEn => kExtCalibEn, 
      kExtCmdInterfaceEn => kExtCmdInterfaceEn,
      kCh1HgMultCoefStatic => kCh1HgMultCoefStatic,
      kCh1HgAddCoefStatic => kCh1HgAddCoefStatic,
      kCh2HgMultCoefStatic => kCh2HgMultCoefStatic,
      kCh2HgAddCoefStatic => kCh2HgAddCoefStatic,
      kCDCEI2C_Addr => kCDCEI2C_Addr,
      kCDCE_SimulationConfig => kCDCE_SimulationConfig,
      kCDCE_SimulationCmdTotal => kCDCE_SimulationCmdTotal,
      kCDCEFreqSel => kCDCEFreqSel_110 
   );
   
InstTbTestTop_ZmodDigitizer_120: entity work.tb_TestTop
   Generic Map(
      kZmodID => kZmodDigitizer1430_125,
      kADC_SamplingClkPeriod => kADC_SamplingClkPeriod_120,
      kADC_ClkDiv => 1,
      kExtCalibEn => kExtCalibEn, 
      kExtCmdInterfaceEn => kExtCmdInterfaceEn,
      kCh1HgMultCoefStatic => kCh1HgMultCoefStatic,
      kCh1HgAddCoefStatic => kCh1HgAddCoefStatic,
      kCh2HgMultCoefStatic => kCh2HgMultCoefStatic,
      kCh2HgAddCoefStatic => kCh2HgAddCoefStatic,
      kCDCEI2C_Addr => kCDCEI2C_Addr,
      kCDCE_SimulationConfig => kCDCE_SimulationConfig,
      kCDCE_SimulationCmdTotal => kCDCE_SimulationCmdTotal,
      kCDCEFreqSel => kCDCEFreqSel_120 
   );
   
InstTbTestTop_ZmodDigitizer_125: entity work.tb_TestTop
   Generic Map(
      kZmodID => kZmodDigitizer1430_125,
      kADC_SamplingClkPeriod => kADC_SamplingClkPeriod_125,
      kADC_ClkDiv => 1,
      kExtCalibEn => kExtCalibEn, 
      kExtCmdInterfaceEn => kExtCmdInterfaceEn,
      kCh1HgMultCoefStatic => kCh1HgMultCoefStatic,
      kCh1HgAddCoefStatic => kCh1HgAddCoefStatic,
      kCh2HgMultCoefStatic => kCh2HgMultCoefStatic,
      kCh2HgAddCoefStatic => kCh2HgAddCoefStatic,
      kCDCEI2C_Addr => kCDCEI2C_Addr,
      kCDCE_SimulationConfig => kCDCE_SimulationConfig,
      kCDCE_SimulationCmdTotal => kCDCE_SimulationCmdTotal,
      kCDCEFreqSel => kCDCEFreqSel_125 
   );         

end Behavioral;